-------------------------------------------------------
--! @file hchecker.vhd
--! @brief Distance-3 Hamming Code checker
--! @author Edson S. Gomi (gomi@usp.br)
--! @date 2019-04-09
-------------------------------------------------------

entity hchecker is
  port (
    DIN :  in bit_vector  (7 downto 1); -- Data in
    DOUT : out bit_vector (7 downto 1); -- Data out
    EBIT : out bit_vector (2 downto 0); -- Bit Number
    ERR :  out bit                      -- Error 
    );
end entity hchecker;

-- Code (7,3) 7 bits, 3 parity bits, 4 data bits
-- DOUT is 7 bit: i7 i6 i5 p4 i3 p2 p1
-- i7 i6 i5 and i3 are data bits
-- p4 p2 and p1 are parity bits   

architecture hchecker_arch of hchecker is

  component parity4 is
    port (
      I :    in bit_vector (3 downto 0);
      ODD :  out bit;
      EVEN : out bit      
      );    
  end component parity4;
  
  component dec3to8 is
  port (
    EN : in  bit;
    I :  in  bit_vector (2 downto 0);
    Y :  out bit_vector (7 downto 0)
    );
  end component dec3to8;

  signal enable :   bit;
  signal i7,i6,i5,i3 : bit;
  signal p4,p2,p1 : bit;
  signal check2:    bit_vector (3 downto 0);
  signal check1:    bit_vector (3 downto 0);
  signal check0:    bit_vector (3 downto 0);  
  signal syndrome : bit_vector (2 downto 0);
  signal corr :     bit_vector (7 downto 0);
  
begin
  -- Components instantiation
  PAR2 : component parity4 port map (check2,syndrome(2),open);
  PAR1 : component parity4 port map (check1,syndrome(1),open);
  PAR0:  component parity4 port map (check0,syndrome(0),open);
  DEC :  component dec3to8 port map (enable,syndrome,corr);
  
  -- Signals instantiation
  enable <= '1';

  i7 <= DIN(7);
  i6 <= DIN(6);
  i5 <= DIN(5);
  i3 <= DIN(3);

  p4 <= DIN(4);
  p2 <= DIN(2);
  p1 <= DIN(1);

  check2 <= i7 & i6 & i5 & p4;
  check1 <= i7 & i6 & i3 & p2;
  check0 <= i7 & i5 & i3 & p1;
  
  DOUT(7) <= i7 xor corr(7);
  DOUT(6) <= i6 xor corr(6);
  DOUT(5) <= i5 xor corr(5);
  DOUT(4) <= p4 xor corr(4);
  DOUT(3) <= i3 xor corr(3);
  DOUT(2) <= p2 xor corr(2);
  DOUT(1) <= p1 xor corr(1);
  

  ERR <= not(corr(0));
  EBIT <= syndrome;
  
end architecture hchecker_arch;
